@@ -5,7 +5,7 @@ Defines models for power systems concepts:
 """
 
 from optimization import newVar,value,sumVars
-from commonscripts import hours,subset,subsetexcept,drop_case_spaces,getattrL,flatten,unique
+from commonscripts import hours,drop_case_spaces,getattrL,unique
 import config, bidding
 from schedule import FixedSchedule
 import logging
@@ -107,13 +107,13 @@ class Generator(object):
         name='',index=None,bus=None):
         
         vars(self).update(locals()) #load in inputs
-        if index is None: self.index=hash(self)        
+        if index is None: self.index=hash(self)
+        if name in [None, '']: self.name = self.index+1 #1 and up naming     
         if self.rampratemin is None and self.rampratemax is not None: self.rampratemin = -1*self.rampratemax
         
-        self.buildCostModel()
         self._makeEmpties()
         self.isControllable=True
-    def buildCostModel(self):
+    def buildCostModel(self,num_breakpoints):
         ''' create a cost model for bidding with :meth:`~bidding.makeModel` '''
         if getattr(self,'heatratestring',None) is not None: 
             costinputs=dict(polyText=self.heatratestring,multiplier=self.fuelcost)
@@ -121,6 +121,7 @@ class Generator(object):
         else: 
             costinputs=dict(polyText=self.costcurvestring)
             self.fuelcost=1
+        costinputs['num_breakpoints']=num_breakpoints
         self.costModel=bidding.makeModel(minInput=self.Pmin, maxInput=self.Pmax,inputNm='Pg',outputNm='C',**costinputs)
     def _makeEmpties(self): self.u,self.power,self.bid,self.startup,self.shutdown=dict(),dict(),dict(),dict(),dict()
         
@@ -154,11 +155,12 @@ class Generator(object):
             if value(self.u[times.initialTime]) == status: h+=self.initialStatusHours
             return h
         
-    def add_timevars(self,times,dispatch_decommit_allowed=False):
+    def add_timevars(self,times,num_breakpoints=config.default_num_breakpoints,dispatch_decommit_allowed=False):
         '''set up dicts for time-varying optimization variables
         (power,u,startup,shutdown,bid)
         '''
         
+        self.buildCostModel(num_breakpoints=num_breakpoints)
         allvars=[]
         commitment_problem= len(times)>1 or dispatch_decommit_allowed
         for time in times:    
@@ -231,6 +233,10 @@ class Generator(object):
             tEndIndex = len(times)
             min_up_hrs_remaining_init = max(0, (self.u[tInitial]==1) * min(tEndHours, self.minuptime - self.initialStatusHours))
             min_down_hrs_remaining_init = max(0, (self.u[tInitial]==0) * min(tEndHours, self.mindowntime - self.initialStatusHours))
+#            except TypeError:
+#                print self.u[tInitial]
+#                print self.initialStatusHours
+#                raise
             #initial up down time
             if min_up_hrs_remaining_init>0: constraintsD['minuptime_'+iden]= 0==sumVars([(1-self.u[times[t]]) for t in range(0,roundoff(min_up_hrs_remaining_init/times.intervalhrs))])
             if min_down_hrs_remaining_init>0: constraintsD['mindowntime_'+iden]= 0==sumVars([self.u[times[t]] for t in range(0,roundoff(min_down_hrs_remaining_init/times.intervalhrs))])
@@ -291,7 +297,6 @@ class Generator_nonControllable(Generator):
         if power is not None and schedule is None: 
             self.schedule = FixedSchedule(P=power)
         if Pmax is None: self.Pmax = self.schedule.maxvalue
-        self.buildCostModel()
         self.isControllable=False
         
     def P(self,time): return self.schedule.getEnergy(time)
@@ -300,7 +305,9 @@ class Generator_nonControllable(Generator):
         if P is None: P=self.schedule.getEnergy(time) #set default power as first scheduled power output
         self.schedule.P[time]=P
     def getstatus(self,t,times): return dict()
-    def add_timevars(self,times,dispatch_decommit_allowed=False): return []
+    def add_timevars(self,times,num_break_points=config.default_num_breakpoints,dispatch_decommit_allowed=False):
+        self.buildCostModel(num_break_points)
+        return []
     def update_vars(self,times=None,problem=None): return
     def fix_timevars(self,times=None): return
     def cost(self,time): return self.operatingcost(time)
@@ -440,7 +447,7 @@ class Load(object):
     def __int__(self): return self.index
     def iden(self,t):     return str(self)+str(t)
     def benifit(self,time=None): return (self.P(time) - self.schedule.getEnergy(time))*config.cost_loadshedding
-    def shed(self,time): return self.schedule.getEnergy(time)- value(self.P(time))
+    def shed(self,time): return self.schedule.getEnergy(time)- value(self.P(time)) #power_to_energy(value(self.P(time)),time)
     def add_timevars(self,times=None,shedding_allowed=False):
         if shedding_allowed: 
             for t in times: self.dispatched_power[t]=newVar('Pd_{}'.format(self.iden(t)),low=0,high=self.schedule.getEnergy(t))
@@ -502,3 +509,7 @@ class Network(object):
             self.Bmatrix[busTo.index,busFrom.index]+=-1/line.X
         for i in range(0,nB): 
             self.Bmatrix[i,i]=-1*sum(self.Bmatrix[i,:])
+
+def power_to_energy(P,time):
+    return P*time.intervalhrs
+    
\ No newline at end of file
