@@ -4,10 +4,10 @@
     vizualization.
     """
 
-import os,sys,types,logging
+import logging
 from collections import OrderedDict
 
-from commonscripts import flatten,getColumn,transpose,elementwiseAdd, getattrL,hours,within,subset,writeCSV,joindir,replace_all
+from commonscripts import flatten,transpose,elementwiseAdd, getattrL,within,subset,writeCSV,joindir,replace_all
 from schedule import Timelist
 from optimization import value
 import config
@@ -137,7 +137,7 @@ class Solution_ED(Solution):
     
     def info_lines(self,t): pass
     def info_buses(self,t): pass
-    def vizualization(self):
+    def vizualization(self,show_cost_also=True):
         ''' economic dispatch visualization '''
         if not self.solved: return
         t=self.times[0]
@@ -147,10 +147,8 @@ class Solution_ED(Solution):
         gensPlotted,genNames,loadsPlotted,loadNames=[],[],[],[]
         minGen=min(getattrL(generators,'Pmin'))
         maxGen=max(getattrL(generators,'Pmax'))
-        if price is not None: 
-            grayColor='.75'
-            plot.plot([minGen,maxGen],[price,price],'--k',color=grayColor)
-            plot.text(maxGen, price, '{p:0.2f} $/MWh'.format(p=price),color=grayColor,horizontalalignment='right')
+
+        #save a plot of the price space - illustrating equal IC
         for gen in generators:
             if gen.status(t):
                 gensPlotted.append( gen.costModel.plotDeriv(P=value(gen.P(t)),linestyle='-') )
@@ -159,16 +157,48 @@ class Solution_ED(Solution):
             if load.kind=='bidding': 
                 loadsPlotted.append( load.bid[t].plotDeriv(P=value(load.P(t)),linestyle=':') )
                 loadNames.append(load.name)
-        legendGens=plot.legend(gensPlotted, genNames, fancybox=True,title='Generators:',loc='upper right')
+        if price is not None: 
+            grayColor='.75'
+            plot.plot([minGen,maxGen],[price,price],'--k',color=grayColor)
+            plot.text(maxGen, price, '{p:0.2f} $/MWh'.format(p=price),color=grayColor,horizontalalignment='right')
+        
+        plot.xlabel('P [MWh]')        
+        if loadsPlotted:     plot.ylabel('Marginal Cost-Benifit [$/MWh]')
+        else:                plot.ylabel('Marginal Cost [$/MWh]')
+
+        
+        ymin,_ = plot.ylim()
+        if ymin<0: plot.ylim(ymin=0)        
+
+        legendGens=plot.legend(gensPlotted, genNames, fancybox=True,title='Generators:',loc='best')
         if loadsPlotted:
-            legendLoads=plot.legend(loadsPlotted, loadNames, fancybox=True,title='Loads:',loc='upper left')
+            plot.legend(loadsPlotted, loadNames, fancybox=True,title='Loads:',loc='best')
             plot.gca().add_artist(legendGens) #add first legend to the axes manually bcs multiple legends get overwritten
         
-        plot.xlabel('P [MWh]')
-        plot.ylabel('Marginal Cost/Benifit [$/MWh]')
-        plot.ylim(ymin=0)        
+        self.savevisualization(filename='dispatch-price.png')
         
-        self.savevisualization(filename='marginalCostWorth.png')    
+        if show_cost_also:
+            #show a plot of the cost space, illustrating the linearization
+            plot.figure()
+            gensPlotted_price=gensPlotted
+            gensPlotted,genNames,loadsPlotted,loadNames=[],[],[],[]
+            for g,gen in enumerate(generators):
+                if gen.status(t):
+                    gensPlotted.append( gen.costModel.plot(P=value(gen.P(t)),linestyle='-',color=gensPlotted_price[g].get_color()) )
+                    genNames.append(gen.name)
+            for load in loads: 
+                if load.kind=='bidding': 
+                    loadsPlotted.append( load.bid[t].plot(P=value(load.P(t)),linestyle=':') )
+                    loadNames.append(load.name)        
+            plot.xlabel('P [MWh]')
+            if loadsPlotted:     plot.ylabel('Cost-Benifit [$/h]')
+            else:                plot.ylabel('Cost [$/h]')
+            legendGens=plot.legend(gensPlotted, genNames, fancybox=True,title='Generators:',loc='best')
+            if loadsPlotted:
+                plot.legend(loadsPlotted, loadNames, fancybox=True,title='Loads:',loc='best')
+                plot.gca().add_artist(legendGens) #add first legend to the axes manually bcs multiple legends get overwritten
+            
+            self.savevisualization(filename='dispatch.png')        
     def saveCSV(self,filename='dispatch.csv'):
         if not self.solved: return
         t=self.times[0]
@@ -198,7 +228,7 @@ class Solution_OPF(Solution):
         
         pos=nx.spectral_layout(G)
         Pinj=[ndata['Pinj'] for bus,ndata in G.nodes(data=True) if 'Pinj' in ndata]
-        nx.draw(G,node_color=Pinj,pos=pos,node_size=1500,alpha=.7,cmap=plot.cm.RdYlBu,fontsize=30)
+        nx.draw(G,node_color=Pinj,pos=pos,node_size=1500,alpha=.7,cmap=plot.cm.get_cmap('RdYlBu'),fontsize=30)
         cb=plot.colorbar(shrink=.8)
         cb.set_label('injected power [MW]',fontsize=15)
         
@@ -233,6 +263,7 @@ class Solution_UC(Solution):
     def info_buses(self,t): pass
     def saveCSV(self,filename='commitment.csv'): 
         if not self.solved: return
+        #times=getattr(self.times,'non_overlap_times',self.times)
         times=self.times
         
         fields,data=[],[]
@@ -324,7 +355,7 @@ class Solution_UC(Solution):
         
         convert_to_GW=True if max(stackBottom)>20000 else False
         
-        for d,load in enumerate(loads):
+        for load in loads:
             color='.8' #gray
             if load.kind in ['shifting','bidding']:
                 Pd=[value(load.P(t)) for t in times]
@@ -339,15 +370,16 @@ class Solution_UC(Solution):
         #show prices
         if withPrices and any(prices):
             prices=replace_all(prices, config.cost_loadshedding, None)
-            axesPrice = plot.axes([figLeft,.75,figWidth,.2],sharex=ax)
-            plt=axesPrice.step(T[1:]+[times.End],prices+[prices[-1]],  where='post') #start from 1 past initial time
-            axesPrice.set_ylabel('price\n[$/MWh]',ha='center',**bigFont)
-            axesPrice.yaxis.set_label_coords(**yLabel_pos)
-            plot.setp(axesPrice.get_xticklabels(), visible=False)
-            #format the price axis nicely
             prices_wo_none=[p for p in prices if p is not None]
-            plot.ylim((.9*min(prices_wo_none),1.1*max(prices_wo_none)))
-            axesPrice.yaxis.set_major_locator(matplotlib.ticker.MaxNLocator(5))
+            if prices_wo_none:
+                axesPrice = plot.axes([figLeft,.75,figWidth,.2],sharex=ax)
+                plt=axesPrice.step(T[1:]+[times.End],prices+[prices[-1]],  where='post') #start from 1 past initial time
+                axesPrice.set_ylabel('price\n[$/MWh]',ha='center',**bigFont)
+                axesPrice.yaxis.set_label_coords(**yLabel_pos)
+                plot.setp(axesPrice.get_xticklabels(), visible=False)
+                #format the price axis nicely
+                plot.ylim((.9*min(prices_wo_none),1.1*max(prices_wo_none)))
+                axesPrice.yaxis.set_major_locator(matplotlib.ticker.MaxNLocator(5))
         
         ax.xaxis_date()
         plottedL=loadsPlotted[::-1]+gensPlotted[::-1]
@@ -383,9 +415,9 @@ class Solution_SCUC(Solution):
     def vizualization(self): raise NotImplementedError('need to implement vizualization for SCUC')
     def saveCSV(self,filename='commitment.csv'): raise NotImplementedError
 class Solution_multistageUC(Solution_UC):
-    def __init__(self,problemsL,times,stageTimes,buses,datadir='.'):
+    def __init__(self,problemsL,times,stageTimes,buses,datadir='.',overlap_hours=0):
         vars(self).update(locals())
-        self.times=Timelist(flatten([list(tL) for tL in stageTimes]))
+        self.times=Timelist(flatten([list(times.non_overlap_times) for times in stageTimes]))
         self.times.setInitial(stageTimes[0].initialTime)
         
         if all([p['status'][0]==1 for p in problemsL]): 
@@ -408,7 +440,7 @@ class Solution_multistageUC(Solution_UC):
         except ZeroDivisionError: self.costerror=0       
     def calcPrices(self):    
         for n,problem in enumerate(self.problemsL):
-            for t in self.stageTimes[n]:
+            for t in self.stageTimes[n].non_overlap_times:
                 for bus in self.buses:
                     bus.price[t] = problem[t]['price_'+bus.iden(t)]
     def show(self):
@@ -416,7 +448,6 @@ class Solution_multistageUC(Solution_UC):
         if not self.solved: return
         self.info_cost()
         self.info_shedding()
-        self.vizualization()
     def info_shedding(self):
         if self.load_shed:
             print 'total load shed={}MW'.format(self.load_shed)
@@ -424,21 +455,21 @@ class Solution_multistageUC(Solution_UC):
         if self.solved: print('{stat} in total of {time:0.4f} sec'.format(stat=self.status,time=self.solveTime))
         else: print(self.solveStatus)
 
-def get_stage_solution(problem,buses,times):
+def get_stage_solution(problem,buses,times,overlap_hours=0):
     solution=dict()
     solution['objective']=float(value(problem.objective))
     solution['solve-time']=problem.solutionTime
     solution['status'] = ( problem.status,problem.statusText )
-    solution['fuelcost_generation']=sum(flatten(flatten([[[value(gen.operatingcost(t)) for t in times] for gen in bus.generators] for bus in buses]) ))
-    solution['truecost_generation']=sum(flatten(flatten([[[value(gen.truecost(t))      for t in times] for gen in bus.generators] for bus in buses]) ))
+    solution['fuelcost_generation']=sum(flatten(flatten([[[value(gen.operatingcost(t)) for t in times.non_overlap_times] for gen in bus.generators] for bus in buses]) ))
+    solution['truecost_generation']=sum(flatten(flatten([[[value(gen.truecost(t))      for t in times.non_overlap_times] for gen in bus.generators] for bus in buses]) ))
     solution['load_shed']=0
 
-    #reduce memory by setting variables to their value (instead of pulp object)
+    #reduce memory by setting variables to their value (instead of full Variable object)
     for bus in buses:
         for gen in bus.generators: gen.fix_vars(times,problem)
         for load in bus.loads: load.update_vars(times,problem)
     
-    for t in times:
+    for t in times.non_overlap_times:
         sln=dict()
         for bus in buses: 
             sln['price_'+bus.iden(t)]=bus.getprice(t,problem)            
